#include <stdio.h>
#include <string.h>
#include <freertos/FreeRTOS.h>
#include <freertos/task.h>
#include <tcpip_adapter.h>
#include <esp_eth.h>
#include <esp_event.h>
#include <esp_log.h>
#include <driver/gpio.h>
#include <sdkconfig.h>

// Expressif: MDC 23, MDIO 18, RST 5, ADDR 1, IP101 1
// wESAP32: MDC 16, MDIO 17, RST -1, ADDR 0, RTL8201

#define CONFIG_EXAMPLE_ETH_MDC_GPIO 16
#define CONFIG_EXAMPLE_ETH_MDIO_GPIO 17
#define CONFIG_EXAMPLE_ETH_PHY_RST_GPIO -1
#define CONFIG_EXAMPLE_ETH_PHY_ADDR 0

static const char *TAG = "eth";
static void ethF_reset_msgs();

// We preset these in case getters are called prior to ethernet initialization.
static char ethG_ip_str[17];
static char ethG_ip_netmask[17];
static char ethG_mac_str[17];
static char ethG_ip_gw[17];

static const char *msg = "DHCP: Discover";

const char *ethFget_ip_str()
{
    return ethG_ip_str;
}

const char *ethFget_mac_str()
{
    return ethG_mac_str;
}

const char *ethFget_ip_netmask_str()
{
    return ethG_ip_netmask;
}

const char *ethFget_ip_gateway_str()
{
    return ethG_ip_gw;
}

/** Event handler for Ethernet events */
static void eth_event_handler(void *arg, esp_event_base_t event_base,
                              int32_t event_id, void *event_data)
{
    uint8_t mac_addr[6] = {0};
    /* we can get the ethernet driver handle from event data */
    esp_eth_handle_t eth_handle = *(esp_eth_handle_t *)event_data;

    switch (event_id)
    {
    case ETHERNET_EVENT_CONNECTED:

        /*
        tcpip_adapter_dhcpc_stop(TCPIP_ADAPTER_IF_ETH);
        tcpip_adapter_ip_info_t eth;
        eth.ip.addr = ipaddr_addr("192.168.3.10");
        eth.netmask.addr = ipaddr_addr("255.255.255.0");
        eth.gw .addr=ipaddr_addr("192.168.3.1");
        tcpip_adapter_set_ip_info(TCPIP_ADAPTER_IF_ETH,(const char*) &eth);
        */

        esp_eth_ioctl(eth_handle, ETH_CMD_G_MAC_ADDR, mac_addr);

        snprintf(ethG_mac_str, sizeof(ethG_mac_str), "%02x%02x%02x%02x%02x%02x",
                 mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);

        ESP_LOGI(TAG, "Ethernet Link Up");
        ESP_LOGI(TAG, "Ethernet HW Addr %02x:%02x:%02x:%02x:%02x:%02x",
                 mac_addr[0], mac_addr[1], mac_addr[2], mac_addr[3], mac_addr[4], mac_addr[5]);

        snprintf(ethG_ip_str, sizeof(ethG_ip_str), msg);
        break;
    case ETHERNET_EVENT_DISCONNECTED:
        ESP_LOGI(TAG, "Ethernet Link Down");
        ethF_reset_msgs();
            snprintf(ethG_ip_str, sizeof(ethG_ip_str), "Link Down");
        break;
    case ETHERNET_EVENT_START:
        ESP_LOGI(TAG, "Ethernet Started");
        break;
    case ETHERNET_EVENT_STOP:
        ESP_LOGI(TAG, "Ethernet Stopped");
        ethF_reset_msgs();
            snprintf(ethG_ip_str, sizeof(ethG_ip_str), "Link Down");
        break;
    default:
        break;
    }
}

static void ethF_reset_msgs()
{
    ethG_ip_str[0] = 0;
    ethG_ip_netmask[0] = 0;
    ethG_ip_gw[0] = 0;
    ethG_mac_str[0] = 0;

    // bzero(ethGip_str,sizeof(ethGip_str));
    // bzero(ethGip_netmask, sizeof(ethGip_netmask));
    // bzero(ethGip_gw, sizeof(ethGip_gw));
}

/** Event handler for IP_EVENT_ETH_GOT_IP */
static void got_ip_event_handler(void *arg, esp_event_base_t event_base,
                                 int32_t event_id, void *event_data)
{
    ip_event_got_ip_t *event = (ip_event_got_ip_t *)event_data;
    const tcpip_adapter_ip_info_t *ip_info = &event->ip_info;

    snprintf(ethG_ip_str, sizeof(ethG_ip_str), IPSTR, IP2STR(&ip_info->ip));
    snprintf(ethG_ip_netmask, sizeof(ethG_ip_netmask), IPSTR, IP2STR(&ip_info->netmask));
    snprintf(ethG_ip_gw, sizeof(ethG_ip_gw), IPSTR, IP2STR(&ip_info->gw));

    ESP_LOGI(TAG, "Ethernet Got IP Address");
    ESP_LOGI(TAG, "~~~~~~~~~~~");
    ESP_LOGI(TAG, "ETHIP:" IPSTR, IP2STR(&ip_info->ip));
    ESP_LOGI(TAG, "ETHMASK:" IPSTR, IP2STR(&ip_info->netmask));
    ESP_LOGI(TAG, "ETHGW:" IPSTR, IP2STR(&ip_info->gw));
    ESP_LOGI(TAG, "~~~~~~~~~~~");
}

void ethFinit()
{
    tcpip_adapter_init();

    ethF_reset_msgs();
    snprintf(ethG_ip_str, sizeof(ethG_ip_str), "Link Down");

    ethG_ip_str[0] = 0;
    ethG_ip_netmask[0] = 0;
    ethG_ip_gw[0] = 0;
    ethG_mac_str[0] = 0;

    ESP_ERROR_CHECK(esp_event_loop_create_default());
    ESP_ERROR_CHECK(tcpip_adapter_set_default_eth_handlers());
    ESP_ERROR_CHECK(esp_event_handler_register(ETH_EVENT, ESP_EVENT_ANY_ID, &eth_event_handler, NULL));
    ESP_ERROR_CHECK(esp_event_handler_register(IP_EVENT, IP_EVENT_ETH_GOT_IP, &got_ip_event_handler, NULL));

    eth_mac_config_t mac_config = ETH_MAC_DEFAULT_CONFIG();
    eth_phy_config_t phy_config = ETH_PHY_DEFAULT_CONFIG();
    phy_config.phy_addr = CONFIG_EXAMPLE_ETH_PHY_ADDR;
    phy_config.reset_gpio_num = CONFIG_EXAMPLE_ETH_PHY_RST_GPIO;

    mac_config.smi_mdc_gpio_num = CONFIG_EXAMPLE_ETH_MDC_GPIO;
    mac_config.smi_mdio_gpio_num = CONFIG_EXAMPLE_ETH_MDIO_GPIO;
    esp_eth_mac_t *mac = esp_eth_mac_new_esp32(&mac_config);

    // esp_eth_phy_t *phy = esp_eth_phy_new_ip101(&phy_config);
    esp_eth_phy_t *phy = esp_eth_phy_new_rtl8201(&phy_config);

    esp_eth_config_t config = ETH_DEFAULT_CONFIG(mac, phy);
    esp_eth_handle_t eth_handle = NULL;
    ESP_ERROR_CHECK(esp_eth_driver_install(&config, &eth_handle));
    ESP_ERROR_CHECK(esp_eth_start(eth_handle));
}